unit QGameEditor;

interface

uses
  SysUtils, Windows, Messages, Classes, Contnrs, 
  Qt, QGraphics, QControls, QForms, QDialogs,
  QGame;

const 
  MAX_GME_WIDTH = 32;
  MAX_GME_HEIGHT = 32;
  
type
  TTipoPonto = (tpNone, tpDoor, tpSpace, tpBall, tpWall);

  TEdGamePoint = record
      Tipo: TTipoPonto;
      Numero: Byte;
  end;
  
type
  TGameEditor = class (TCustomControl)
  private
    Fase: Array [0..MAX_GME_WIDTH, 0..MAX_GME_HEIGHT] Of TEdGamePoint;
    FColunas: Integer;
    FLinhas: Integer;
    FNumeroObjeto: Byte;
    FTipoObjeto: TTipoPonto;
    procedure SetColunas(Value: Integer);
    procedure SetLinhas(Value: Integer);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: 
            Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Clear;
    procedure LoadFromFile(const AFileName: String); virtual;
    procedure SaveToFile(const AFileName: String); virtual;
    property NumeroObjeto: Byte read FNumeroObjeto write FNumeroObjeto;
    property TipoObjeto: TTipoPonto read FTipoObjeto write FTipoObjeto;
  published
    property Colunas: Integer read FColunas write SetColunas;
    property Linhas: Integer read FLinhas write SetLinhas;
  end;
  

procedure Register;

implementation

uses Math;

procedure Register;
begin
end;

{
********************************* TGameEditor **********************************
}
constructor TGameEditor.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  //
  Color := clLime;
end;

procedure TGameEditor.Clear;
var
  X, Y: Integer;
begin
  for X := 0 to MAX_GME_WIDTH do
      for Y := 0 to MAX_GME_HEIGHT do begin
          Fase[X, Y].Tipo   := tpNone;
          Fase[X, Y].Numero := 0;
      end;
end;

procedure TGameEditor.LoadFromFile(const AFileName: String);
var
  S: TStrings;
  I, J: Integer;
begin
  Clear;
  S := TStringList.Create;
  S.LoadFromFile(AFileName);
  //Remover cabecalho e informaes de como concluir o jogo
  for  i := 0 to S.Count - 1 do begin
       for  j := 1 to Length(S[i]) div 2 do begin
            Colunas := Max(Colunas, J);
            Linhas  := Max(Linhas, I+1);
            Case  S[i][(j*2)-1] Of
                  'W': Fase[J-1, I].Tipo := tpWall;
                  'S': Fase[J-1, I].Tipo := tpSpace;
                  'B': Fase[J-1, I].Tipo := tpBall;
                  'D': Fase[J-1, I].Tipo := tpDoor;
            end;
            Fase[J-1, I].Numero := StrToInt(S[i][j*2]);
       end;
  end;
  S.Free;
end;

procedure TGameEditor.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: 
        Integer);
var
  AX, AY: Integer;
begin
  inherited MouseDown(Button, Shift, X, Y);
  AX := X div TGamePoint.Size;
  AY := Y div TGamePoint.Size;
  Fase[AX, AY].Numero := NumeroObjeto;
  Fase[AX, AY].Tipo   := TipoObjeto;
    Invalidate;
end;

procedure TGameEditor.Paint;
var
  Lin: Integer;
  Col: Integer;
  R: TRect;
begin
  inherited Paint;
  for  Col := 0 to Colunas-1 do
       for  Lin := 0 to Linhas-1 do
            with Canvas do begin
                 Brush.Color := GetColor(Fase[Col, Lin].Numero);
                 Pen.Color := clWhite;
                 R := Rect(Col * TGamePoint.Size, Lin * TGamePoint.Size,
                           Col * TGamePoint.Size + TGamePoint.Size,
                           Lin * TGamePoint.Size + TGamePoint.Size);
                 Case Fase[Col, Lin].Tipo Of
                      tpNone: Rectangle(R);
                      tpWall: begin
                               Pen.Color := clRed;
                               Rectangle(R);
                              end;
                      tpSpace: Rectangle(R);
                      tpBall: Ellipse(R);
                      tpDoor: begin
                                Pen.Color := clLime;
                                Rectangle(R);
                                MoveTo(R.Left, R.Top);
                                LineTo(R.Right, R.Bottom);
  
                                MoveTo(R.Right, R.Top);
                                LineTo(R.Left ,R.Bottom);
                              end;
                 end;
            end;
end;

procedure TGameEditor.SaveToFile(const AFileName: String);
var
  S: TStrings;
  I, J: Integer;
begin
  S := TStringList.Create;
  //Remover cabecalho e informaes de como concluir o jogo
  for  i := 0 to Linhas-1 do begin
       S.Add('');
       for  j := 0 to Colunas-1 do begin
            Case  Fase[J, I].Tipo Of
                  tpWall:  S[i] := S[i] + 'W';
                  tpNone,
                  tpSpace: S[i] := S[i] + 'S';
                  tpBall:  S[i] := S[i] + 'B';
                  tpDoor:  S[i] := S[i] + 'D';
            end;
            S[i] := S[i] + IntToStr(Fase[J, I].Numero);
       end;
  end;
  S.SaveToFile(AFileName);
  S.Free;
end;

procedure TGameEditor.SetColunas(Value: Integer);
begin
  if FColunas <> Value then
  begin
  Width := Value * TGamePoint.Size;
  Invalidate;
    FColunas := Value;
  end;
end;

procedure TGameEditor.SetLinhas(Value: Integer);
begin
  if FLinhas <> Value then
  begin
  Height := Value * TGamePoint.Size;
  Invalidate;
    FLinhas := Value;
  end;
end;


end.
